@@ -30,7 +30,7 @@
 #include "prim_internal.h"
 #include "prim_YUV.h"
 
-#if defined(NEON_ENABLED)
+#if defined(NEON_INTRINSICS_ENABLED)
 #include <arm_neon.h>
 
 static primitives_t* generic = NULL;
@@ -81,13 +81,13 @@ static INLINE BYTE* neon_YuvToRgbPixel(BYTE* pRGB, int16x8_t Y, int16x8_t D, int
                                        const uint8_t rPos, const uint8_t gPos, const uint8_t bPos,
                                        const uint8_t aPos)
 {
-	uint8x8x4_t bgrx;
 	const int32x4_t Ch = vmulq_n_s32(vmovl_s16(vget_high_s16(Y)), 256); /* Y * 256 */
 	const int32x4_t Cl = vmulq_n_s32(vmovl_s16(vget_low_s16(Y)), 256);  /* Y * 256 */
 	const int16x4_t Dh = vget_high_s16(D);
 	const int16x4_t Dl = vget_low_s16(D);
 	const int16x4_t Eh = vget_high_s16(E);
 	const int16x4_t El = vget_low_s16(E);
+	uint8x8x4_t bgrx = vld4_u8(pRGB);
 	{
 		/* B = (256L * Y + 475 * (U - 128)) >> 8*/
 		const int16x4_t c475 = vdup_n_s16(475);
@@ -121,10 +121,6 @@ static INLINE BYTE* neon_YuvToRgbPixel(BYTE* pRGB, int16x8_t Y, int16x8_t D, int
 		const int16x8_t R = vcombine_s16(vqmovn_s32(Rl), vqmovn_s32(Rh));
 		bgrx.val[rPos] = vqmovun_s16(R);
 	}
-	{
-		/* A */
-		bgrx.val[aPos] = vdup_n_u8(0xFF);
-	}
 	vst4_u8(pRGB, bgrx);
 	pRGB += 32;
 	return pRGB;
@@ -213,7 +209,6 @@ static INLINE pstatus_t neon_YUV420ToX(const BYTE* WINPR_RESTRICT pSrc[3], const
 				const BYTE r = YUV2R(Y, U, V);
 				const BYTE g = YUV2G(Y, U, V);
 				const BYTE b = YUV2B(Y, U, V);
-				pRGB1[aPos] = 0xFF;
 				pRGB1[rPos] = r;
 				pRGB1[gPos] = g;
 				pRGB1[bPos] = b;
@@ -226,7 +221,6 @@ static INLINE pstatus_t neon_YUV420ToX(const BYTE* WINPR_RESTRICT pSrc[3], const
 				const BYTE r = YUV2R(Y, U, V);
 				const BYTE g = YUV2G(Y, U, V);
 				const BYTE b = YUV2B(Y, U, V);
-				pRGB2[aPos] = 0xFF;
 				pRGB2[rPos] = r;
 				pRGB2[gPos] = g;
 				pRGB2[bPos] = b;
@@ -325,7 +319,6 @@ static INLINE pstatus_t neon_YUV444ToX(const BYTE* WINPR_RESTRICT pSrc[3], const
 			const BYTE r = YUV2R(Y, U, V);
 			const BYTE g = YUV2G(Y, U, V);
 			const BYTE b = YUV2B(Y, U, V);
-			pRGB[aPos] = 0xFF;
 			pRGB[rPos] = r;
 			pRGB[gPos] = g;
 			pRGB[bPos] = b;
@@ -547,7 +540,7 @@ static pstatus_t neon_ChromaV1ToYUV444(const BYTE* WINPR_RESTRICT pSrcRaw[3],
 	const UINT32 oddY = 1;
 	const UINT32 evenY = 0;
 	const UINT32 oddX = 1;
-	/* The auxilary frame is aligned to multiples of 16x16.
+	/* The auxiliary frame is aligned to multiples of 16x16.
 	 * We need the padded height for B4 and B5 conversion. */
 	const UINT32 padHeigth = nHeight + 16 - nHeight % 16;
 	const UINT32 halfPad = halfWidth % 16;
@@ -741,7 +734,7 @@ static pstatus_t neon_YUV420CombineToYUV444(avc444_frame_type type,
 
 void primitives_init_YUV_neon(primitives_t* WINPR_RESTRICT prims)
 {
-#if defined(NEON_ENABLED)
+#if defined(NEON_INTRINSICS_ENABLED)
 	generic = primitives_get_generic();
 	primitives_init_YUV(prims);
 
@@ -753,7 +746,7 @@ void primitives_init_YUV_neon(primitives_t* WINPR_RESTRICT prims)
 		prims->YUV420CombineToYUV444 = neon_YUV420CombineToYUV444;
 	}
 #else
-	WLog_VRB(PRIM_TAG, "undefined WITH_NEON");
+	WLog_VRB(PRIM_TAG, "undefined WITH_SIMD or neon intrinsics not available");
 	WINPR_UNUSED(prims);
 #endif
 }
