@@ -47,7 +47,6 @@
 #if defined(WITH_SMARTCARD_EMULATE)
 #include <freerdp/emulate/scard/smartcard_emulate.h>
 
-#define str(x) #x
 #define wrap(ctx, fkt, ...)                                             \
 	ctx->useEmulatedCard ? Emulate_##fkt(ctx->emulation, ##__VA_ARGS__) \
 	                     : ctx->pWinSCardApi->pfn##fkt(__VA_ARGS__)
@@ -191,6 +190,9 @@ static LONG smartcard_ListReaderGroupsA_Call(scard_call_context* smartcard, wStr
 	cchGroups = SCARD_AUTOALLOCATE;
 	ret.ReturnCode =
 	    wrap(smartcard, SCardListReaderGroupsA, operation->hContext, (LPSTR)&mszGroups, &cchGroups);
+	if (cchGroups == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
+
 	ret.msz = (BYTE*)mszGroups;
 	ret.cBytes = cchGroups;
 
@@ -220,8 +222,15 @@ static LONG smartcard_ListReaderGroupsW_Call(scard_call_context* smartcard, wStr
 	cchGroups = SCARD_AUTOALLOCATE;
 	status = ret.ReturnCode = wrap(smartcard, SCardListReaderGroupsW, operation->hContext,
 	                               (LPWSTR)&mszGroups, &cchGroups);
+	if (cchGroups == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
+
 	ret.msz = (BYTE*)mszGroups;
-	ret.cBytes = cchGroups * sizeof(WCHAR);
+
+	WINPR_ASSERT(cchGroups < SCARD_AUTOALLOCATE / sizeof(WCHAR));
+	const size_t blen = sizeof(WCHAR) * cchGroups;
+	WINPR_ASSERT(blen <= UINT32_MAX);
+	ret.cBytes = (UINT32)blen;
 
 	if (status != SCARD_S_SUCCESS)
 		return status;
@@ -326,41 +335,33 @@ static DWORD filter_device_by_name_w(wLinkedList* list, LPWSTR* mszReaders, DWOR
 static LONG smartcard_ListReadersA_Call(scard_call_context* smartcard, wStream* out,
                                         SMARTCARD_OPERATION* operation)
 {
-	LONG status = 0;
 	ListReaders_Return ret = { 0 };
 	LPSTR mszReaders = NULL;
-	DWORD cchReaders = 0;
-	ListReaders_Call* call = NULL;
 
 	WINPR_ASSERT(smartcard);
 	WINPR_ASSERT(out);
 	WINPR_ASSERT(operation);
 
-	call = &operation->call.listReaders;
-	cchReaders = SCARD_AUTOALLOCATE;
-	status = ret.ReturnCode = wrap(smartcard, SCardListReadersA, operation->hContext,
-	                               (LPCSTR)call->mszGroups, (LPSTR)&mszReaders, &cchReaders);
+	ListReaders_Call* call = &operation->call.listReaders;
+	DWORD cchReaders = SCARD_AUTOALLOCATE;
+	LONG status = ret.ReturnCode = wrap(smartcard, SCardListReadersA, operation->hContext,
+	                                    (LPCSTR)call->mszGroups, (LPSTR)&mszReaders, &cchReaders);
+	if (cchReaders == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
 
 	if (status != SCARD_S_SUCCESS)
-	{
 		return scard_log_status_error(TAG, "SCardListReadersA", status);
-	}
 
 	cchReaders = filter_device_by_name_a(smartcard->names, &mszReaders, cchReaders);
 	ret.msz = (BYTE*)mszReaders;
 	ret.cBytes = cchReaders;
 
 	status = smartcard_pack_list_readers_return(out, &ret, FALSE);
-	if (status != SCARD_S_SUCCESS)
-	{
-		return scard_log_status_error(TAG, "smartcard_pack_list_readers_return", status);
-	}
-
 	if (mszReaders)
 		wrap(smartcard, SCardFreeMemory, operation->hContext, mszReaders);
 
 	if (status != SCARD_S_SUCCESS)
-		return status;
+		return scard_log_status_error(TAG, "smartcard_pack_list_readers_return", status);
 
 	return ret.ReturnCode;
 }
@@ -395,6 +396,8 @@ static LONG smartcard_ListReadersW_Call(scard_call_context* smartcard, wStream*
 	cchReaders = SCARD_AUTOALLOCATE;
 	status = ret.ReturnCode = wrap(smartcard, SCardListReadersW, operation->hContext, string.wz,
 	                               (LPWSTR)&mszReaders.pw, &cchReaders);
+	if (cchReaders == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
 
 	if (status != SCARD_S_SUCCESS)
 		return scard_log_status_error(TAG, "SCardListReadersW", status);
@@ -850,6 +853,8 @@ static LONG smartcard_GetReaderIcon_Call(scard_call_context* smartcard, wStream*
 	ret.ReturnCode = wrap(smartcard, SCardGetReaderIconW, operation->hContext, call->szReaderName,
 	                      (LPBYTE)&ret.pbData, &ret.cbDataLen);
 	scard_log_status_error(TAG, "SCardGetReaderIconW", ret.ReturnCode);
+	if (ret.cbDataLen == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
 
 	status = smartcard_pack_get_reader_icon_return(out, &ret);
 	wrap(smartcard, SCardFreeMemory, operation->hContext, ret.pbData);
@@ -1223,6 +1228,9 @@ static LONG smartcard_StatusA_Call(scard_call_context* smartcard, wStream* out,
 	         &ret.dwState, &ret.dwProtocol, cbAtrLen ? (BYTE*)&ret.pbAtr : NULL, &cbAtrLen);
 
 	scard_log_status_error(TAG, "SCardStatusA", status);
+	if (cchReaderLen == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
+
 	if (status == SCARD_S_SUCCESS)
 	{
 		if (!call->fmszReaderNamesIsNULL)
@@ -1275,6 +1283,9 @@ static LONG smartcard_StatusW_Call(scard_call_context* smartcard, wStream* out,
 	         call->fmszReaderNamesIsNULL ? NULL : (LPWSTR)&mszReaderNames, &ret.cBytes,
 	         &ret.dwState, &ret.dwProtocol, (BYTE*)&ret.pbAtr, &cbAtrLen);
 	scard_log_status_error(TAG, "SCardStatusW", status);
+	if (ret.cBytes == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
+
 	if (status == SCARD_S_SUCCESS)
 	{
 		if (!call->fmszReaderNamesIsNULL)
@@ -1284,7 +1295,10 @@ static LONG smartcard_StatusW_Call(scard_call_context* smartcard, wStream* out,
 	}
 
 	/* SCardStatusW returns number of characters, we need number of bytes */
-	ret.cBytes *= sizeof(WCHAR);
+	WINPR_ASSERT(ret.cBytes < SCARD_AUTOALLOCATE / sizeof(WCHAR));
+	const size_t blen = sizeof(WCHAR) * ret.cBytes;
+	WINPR_ASSERT(blen <= UINT32_MAX);
+	ret.cBytes = (UINT32)blen;
 
 	status = smartcard_pack_status_return(out, &ret, TRUE);
 	if (status != SCARD_S_SUCCESS)
@@ -1401,6 +1415,9 @@ static LONG smartcard_GetAttrib_Call(scard_call_context* smartcard, wStream* out
 	ret.ReturnCode =
 	    wrap(smartcard, SCardGetAttrib, operation->hCard, call->dwAttrId, pbAttr, &cbAttrLen);
 	scard_log_status_error(TAG, "SCardGetAttrib", ret.ReturnCode);
+	if (cbAttrLen == SCARD_AUTOALLOCATE)
+		return SCARD_F_UNKNOWN_ERROR;
+
 	ret.cbAttrLen = cbAttrLen;
 
 	status = smartcard_pack_get_attrib_return(out, &ret, call->dwAttrId, call->cbAttrLen);
@@ -1470,7 +1487,7 @@ static LONG smartcard_LocateCardsByATRA_Call(scard_call_context* smartcard, wStr
 
 	for (UINT32 i = 0; i < call->cReaders; i++)
 	{
-		states[i].szReader = (LPSTR)call->rgReaderStates[i].szReader;
+		states[i].szReader = call->rgReaderStates[i].szReader;
 		states[i].dwCurrentState = call->rgReaderStates[i].dwCurrentState;
 		states[i].dwEventState = call->rgReaderStates[i].dwEventState;
 		states[i].cbAtr = call->rgReaderStates[i].cbAtr;
@@ -1937,7 +1954,7 @@ void smartcard_call_context_free(scard_call_context* ctx)
 	ctx->pWinSCardApi = NULL;
 
 	HashTable_Free(ctx->rgSCardContextList);
-	CloseHandle(ctx->stopEvent);
+	(void)CloseHandle(ctx->stopEvent);
 	free(ctx);
 }
 
